package com.loafle.overflow.commons.dao;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.PersistenceContext;
import java.lang.reflect.ParameterizedType;

/**
 * Created by insanity on 17. 5. 25.
 */
public class JPABaseDAO<T> implements BaseDAO<T>  {

    @PersistenceContext
    private EntityManager entityManager;
    private final Class<T> entityType;

    public JPABaseDAO() {
        this.entityManager = Persistence.createEntityManagerFactory("overflow").createEntityManager();
        this.entityType = ((Class<T>) ((ParameterizedType) getClass().getGenericSuperclass()).getActualTypeArguments()[0]);
    }

    protected EntityManager getEntityManager() {
        return this.entityManager;
    }

    public T find(String id) {
        return this.entityManager.find(this.entityType, Long.parseLong(id));
    }

    public T create(T entity) {
        EntityTransaction tx = this.entityManager.getTransaction();
        if (!tx.isActive()) tx.begin();
        try {
            this.entityManager.persist(entity);
            tx.commit();
        }catch(Exception e) {
            tx.rollback();
            return null;
        }
        return entity;
    }

    public T update(T entity) {
        EntityTransaction tx = this.entityManager.getTransaction();
        if (!tx.isActive()) tx.begin();
        try {
            T result = this.entityManager.merge(entity);
            tx.commit();
            return result;
        }catch(Exception e) {
            tx.rollback();
            return null;
        }
    }

    public String delete(T entity) {
        EntityTransaction tx = this.entityManager.getTransaction();
        if (!tx.isActive()) tx.begin();
        try {
            this.entityManager.remove(entityManager.contains(entity) ? entity : entityManager.merge(entity));
            tx.commit();
            return "Success";
        }catch(Exception e) {
            tx.rollback();
            return "Fail";
        }

    }
}
